namespace JAM
{
	const kStr CREDITS_MAP_PATH = "levels/credits.map";
	
	bool skipCredits;
	bool canShowStartCredits;
	bool showedStartCredits;
	int creditsScriptIDCaller;

	// Convenience function
	bool IsCreditsPlaying()
	{
		return GameVarHasFlags("JAM", GV_JAM_FLAG_CREDITS);
	}
	
	void TickMapCreditsStart()
	{
		//show start credits when ready to
		if (!showedStartCredits && canShowStartCredits && !Player.Locked() && !Camera.Active())
		{
			showedStartCredits = true;
			MapDef@ mapDef = GetMapDef(Game.GetCurrentMapID());
			if (mapDef !is null)
			{
				switch (Game.GetCurrentMapID())
				{
					case 63:
						Game.PrintHelp("<Ravaged Labyrinth>\n\nBy DoomMarine23");
						break;
					case 88:
						Game.PrintHelp("<Arcane Temple>\n\nBy operative lm");
						break;
					case 60:
						Game.PrintHelp("<Atsilv Gulf>\n\nBy Badger");
						break;
					case 65:
						Game.PrintHelp("<Jungle Sanctuary>\n\nBy Smoke39");
						break;
					case 61:
						Game.PrintHelp("<Primordial Vale>\n\nBy BehemothProgrammer");
						break;
				}
			}
		}
	}

	void StartMapCreditsFlyThrough(const int scriptIDCaller)
	{
		creditsScriptIDCaller =	scriptIDCaller;
		
		kActor@ pActor = Player.Actor().CastToActor();
		MapDef@ mapDef = GetMapDef(Game.GetCurrentMapID());
		
		//if map has it's own custom credit script then run that instead
		if (mapDef.customFlyCreditsScriptID != -1)
		{
			Game.CallDelayedMapScript(mapDef.customFlyCreditsScriptID, pActor, 0.0f);
			return;
		}

		pActor.Flags() |= AF_NODRAW;
		Camera.ClearLookAtActor();
		Camera.ClearFinalView();
		Camera.ClearViewTracks();
		Camera.StartCinematic(CMF_LOCK_PLAYER|CMF_UNLOCK_PLAYER_ON_FINISH|CMF_SHOW_HIDDEN_OBJECTS|CMF_NO_INITIAL_FADEOUT|CMF_NO_LETTERBOX);
		Camera.fov = 74.0f;
		Camera.SetFinalView(0);
		
		while (Camera.CinematicState() < 2) { delay(0.0f); } //wait until cinema fades in
		
		Game.CallDelayedMapScript(SCRIPTID_MAP_CREDITSFLYSKIP_TICK, pActor, 0.0f);
		
		float lerpTime = 8.0f;
		int lerpType = CMLT_LINEAR; //CMLT_COSINE
		
		//Get Camera Path Points
		array<kVec3>@ pathPoints;
		if (mapDef is null || mapDef.creditsPath.length() < 4)
			@pathPoints = array<kVec3> = {kVec3(0.0f, 0.0f, 1000.0f), kVec3(0.0f, 1000.0f, 1000.0f), kVec3(0.0f, 1000.0f, 1000.0f), kVec3(0.0f, 1010.0f, 1000.0f)};
		else
			@pathPoints = mapDef.creditsPath;
		
		//Display Credits Text
		if (mapDef !is null)
		{
			float totalCinemaTime = ((int(pathPoints.length()) - 1) / 2) * lerpTime;
			switch (Game.GetCurrentMapID())
			{
				case 63:
					mapDef.PrintCreditLines("By DoomMarine23", "<Ravaged Labyrinth>", totalCinemaTime);
					break;
				case 88:
					mapDef.PrintCreditLines("By operative lm", "<Arcane Temple>", totalCinemaTime);
					break;
				case 60:
					mapDef.PrintCreditLines("By Badger", "<Atsilv Gulf>", totalCinemaTime);
					break;
				case 65:
					mapDef.PrintCreditLines("By Smoke39", "<Jungle Sanctuary>", totalCinemaTime);
					break;
				case 61:
					mapDef.PrintCreditLines("By BehemothProgrammer", "<Primordial Vale>", totalCinemaTime);
					break;
			}
		}
		
		//Play camera position tracks through the path points
		int pointsLength = int(pathPoints.length()) - (int(pathPoints.length()) % 2);
		for (int i = 0; i < pointsLength - 2; i+=2)
		{
			Camera.SetPositionTrack(0, pathPoints[i], pathPoints[i+2], pathPoints[i+1], pathPoints[i+3]);
			Camera.AutoPlayPositionTrack(0, lerpTime, lerpType);
			delay(lerpTime);
		}
		
		WarpToNextCreditsMap();
	}
	
	void WarpToNextCreditsMap()
	{
		int nextCreditsMapIndex;
		GameVariables.GetInt("JAM_CREDITSMAPINDEX", nextCreditsMapIndex);
		nextCreditsMapIndex++;
		GameVariables.SetValue("JAM_CREDITSMAPINDEX", "" + nextCreditsMapIndex);
		if (IndexInRange(CREDITS_MAPS.length(), nextCreditsMapIndex))
		{
			int nextCreditsMapID = CREDITS_MAPS[nextCreditsMapIndex];
			MapDef@ nextMapDef = GetMapDef(nextCreditsMapID);
			if (nextMapDef !is null)
			{
				delay(0.0f); //required. (if map was changed during play) Or warping will not work correctly.
				PlayLoop.StartWarp(Player.Actor().CastToActor(), nextMapDef.creditsWarpTID, nextCreditsMapID);
				return;
			}			
		}
		
		FinishFlyCredits();
	}
	
	void FinishFlyCredits()
	{
		delay(0.0f);
		Game.ChangeMap(CREDITS_MAP_PATH);
	}
	
	void CreditsMapStart(const int scriptIDCaller)
	{
		creditsScriptIDCaller =	scriptIDCaller;
		
		kActor@ pActor = Player.Actor().CastToActor();
		pActor.Flags() |= AF_NODRAW;
		Camera.ClearLookAtActor();
		Camera.ClearFinalView();
		Camera.ClearViewTracks();
		Camera.StartCinematic(CMF_LOCK_PLAYER|CMF_UNLOCK_PLAYER_ON_FINISH|CMF_SHOW_HIDDEN_OBJECTS|CMF_NO_INITIAL_FADEOUT|CMF_NO_LETTERBOX);
		Camera.fov = 74.0f;
		Camera.SetFinalView(0);
		kVec3 camPos(0.0f, 0.0f, 0.0f);
		kVec3 camLookPos(0.0f, 0.0f, 1.0f);
		Camera.SetPositionTrack(0, camPos, camPos, camLookPos, camLookPos);
		Camera.AutoPlayPositionTrack(0, 1.0f, CMLT_LINEAR);
		
		while (Camera.CinematicState() < 2) { delay(0.0f); } //wait until cinema fades in
		
		Game.PrintLine("The 2019 Turok Community Map Jam", 0, 600);
		Game.PrintLine("Thanks For Playing", 1, 600);
		Game.CallDelayedMapScript(SCRIPTID_MAP_CREDITSENDSKIP_TICK, Player.Actor().CastToActor(), 0.0f);
		delay(8.0f);
		Game.PrintLine("Until next time", 0, 300);
		delay(4.0f);
		EndCredits();
	}
	
	void HaltCreditScriptCaller()
	{
		if (creditsScriptIDCaller < 0)
			return;
		
		Game.HaltMapScript(creditsScriptIDCaller);
	}
	
	void EndCredits()
	{
		GameVarSetFlags("JAM", GV_JAM_FLAG_CREDITS, false);
		delay(0.0f);
		Game.Restart();
	}
}
